@@ -25,6 +25,8 @@ from email.utils import parsedate_tz, mktime_tz
 from platform import python_version, version as platform_version
 from random import randint
 from time import time, sleep, gmtime, strftime
+from uuid import uuid4
+from datetime import datetime
 import requests
 import io
 import urllib.parse as urlparse
@@ -173,6 +175,8 @@ class Connection:
                              "actions-queued": 0}
         self.server_status = {"status": "Never contacted",
                               "endpoint": self.endpoint}
+        self.sync_started = False
+        self.sync_ended = False
         if auth:
             self.auth = auth
         elif auth_dict:
@@ -184,6 +188,7 @@ class Connection:
         if user_agent and user_agent.strip():
             ua_string = user_agent.strip() + " " + ua_string
         self.session.headers["User-Agent"] = ua_string
+        self.uuid = str(uuid4())
 
     def _get_auth(self, ims_host, ims_endpoint_jwt,
                   tech_acct_id=None, api_key=None, client_secret=None,
@@ -308,7 +313,7 @@ class Connection:
             if re.result.status_code == 404:
                 if self.logger: self.logger.debug("Ran %s query: %s %s (0 found)",
                                                   object_type, url_params, query_params)
-                return [], True
+                return [], True, 0, 0, 0, 0
             else:
                 raise re
 
@@ -369,7 +374,7 @@ class Connection:
         Execute multiple Actions (each containing commands on a single object).
         Normally, the actions are sent for execution immediately (possibly preceded
         by earlier queued commands), but if you are going for maximum efficiency
-        you can set immeediate=False which will cause the connection to wait
+        you can set immediate=False which will cause the connection to wait
         and batch as many actions as allowed in each server call.
 
         Since any command can fill the current batch, one or more of your commands may be submitted
@@ -431,6 +436,16 @@ class Connection:
             raise BatchError(exceptions, queued, sent, completed)
         return queued, sent, completed
 
+    def start_sync(self):
+        """Signal the beginning of a sync operation
+        Sends a header with the first batch of UMAPI actions"""
+        self.sync_started = True
+
+    def end_sync(self):
+        """Signal the end of a sync operation
+        Sends a header with the next batch of UMAPI actions"""
+        self.sync_ended = True
+
     def _execute_batch(self, actions):
         """
         Execute a single batch of Actions.
@@ -466,20 +481,30 @@ class Connection:
         :param body: (optional) list of dictionaries to be serialized into the request body
         :return: the requests.result object (on 200 response), raise error otherwise
         """
+        extra_headers = {"X-Request-Id": f"{self.uuid}_{int(datetime.now().timestamp()*1000)}"}
+        # if the sync_started or sync_ended flags are set, send a header for any type of call
+        if self.sync_started:
+            self.logger.info("Sending start_sync signal")
+            extra_headers['Pragma'] = 'umapi-sync-start'
+            self.sync_started = False
+        elif self.sync_ended:
+            self.logger.info("Sending end_sync signal")
+            extra_headers['Pragma'] = 'umapi-sync-end'
+            self.sync_ended = False
         if body:
             request_body = json.dumps(body)
             def call():
                 return self.session.post(self.endpoint + path, auth=self.auth, data=request_body, timeout=self.timeout,
-                                         verify=self.ssl_verify)
+                                         verify=self.ssl_verify, headers=extra_headers)
         else:
             if not delete:
                 def call():
                     return self.session.get(self.endpoint + path, auth=self.auth, timeout=self.timeout,
-                                            verify=self.ssl_verify)
+                                            verify=self.ssl_verify, headers=extra_headers)
             else:
                 def call():
                     return self.session.delete(self.endpoint + path, auth=self.auth, timeout=self.timeout,
-                                               verify=self.ssl_verify)
+                                               verify=self.ssl_verify, headers=extra_headers)
 
         start_time = time()
         result = None
@@ -500,7 +525,7 @@ class Connection:
             if checked_result.success:
                 return result
 
-            if self.logger: self.logger.warning("UMAPI timeout...service unavailable (code %s on try %d)",
+            if self.logger: self.logger.warning("UMAPI request limit reached (code %s on try %d)",
                                                 checked_result.status_code, num_attempts)
 
             retry_wait = checked_result.timeout
@@ -511,7 +536,7 @@ class Connection:
 
             if num_attempts < self.retry_max_attempts:
                 if retry_wait > 0:
-                    if self.logger: self.logger.warning("Next retry in %d seconds...", retry_wait)
+                    if self.logger: self.logger.warning("waiting %d seconds to continue...", retry_wait)
                     sleep(retry_wait)
                 else:
                     if self.logger: self.logger.warning("Immediate retry...")
